/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <system_settings.h>
#include <efl_extension.h>
#include <app.h>
#include <dlog.h>
#include "$(appName).h"
#include "view.h"
#include "data.h"

#define CALC_BUTTON 20

/**
 * Pre-declared values of the button.
 */
static button_info_s BUTTON_PARAM[20] = {
	{NULL, KEY_TYPE_NUM, KEY_VAL_ZERO, "0", NULL, "zero_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_ONE, "1", NULL, "one_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_TWO, "2", NULL, "two_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_THREE, "3", NULL, "three_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_FOUR, "4", NULL, "four_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_FIVE, "5", NULL, "five_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_SIX, "6", NULL, "six_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_SEVEN, "7", NULL, "seven_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_EIGHT, "8", NULL, "eight_button"},
	{NULL, KEY_TYPE_NUM, KEY_VAL_NINE, "9", NULL, "nine_button"},
	{NULL, KEY_TYPE_EX, KEY_VAL_RE, "re", NULL, "reset_button"},
	{NULL, KEY_TYPE_OP, KEY_VAL_DIV, "/", NULL, "div_button"},
	{NULL, KEY_TYPE_OP, KEY_VAL_MUL, "*", NULL, "multi_button"},
	{NULL, KEY_TYPE_EX, KEY_VAL_DEL, "del", NULL, "del_button"},
	{NULL, KEY_TYPE_OP, KEY_VAL_MINUS, "-", NULL, "minus_button"},
	{NULL, KEY_TYPE_OP, KEY_VAL_PLUS, "+", NULL, "plus_button"},
	{NULL, KEY_TYPE_EX, KEY_VAL_BLANK, "(", ")", "blank_button"},
	{NULL, KEY_TYPE_EX, KEY_VAL_DECIMAL, ".", NULL, "decimal_button"},
	{NULL, KEY_TYPE_EX, KEY_VAL_PL_MI, "-", NULL, "pl_mi_button"},
	{NULL, KEY_TYPE_EX, KEY_VAL_RESULT, "=", NULL, "result_button"}
};

/**
 * @brief Adds a number to the displayed formula.
 * @param[in] btn_info Information of the pressed button
 */
static void _add_number(button_info_s *btn_info)
{
	char *text = NULL;
	text = data_add_number(btn_info);
	if (text) {
		view_create_text_popup(text);
	}

	if (text) {
		free(text);
	}
}

/**
 * @brief Adds a operand to the displayed formula.
 * @param[in] btn_info Information of the pressed button
 */
static void _add_operand(button_info_s *btn_info)
{

	data_add_operand(btn_info);
}

/**
 * @brief Resets the displayed formula and initializes the resource.
 * @param[in] btn_info Information of the pressed button
 */
static void _extra_reset(button_info_s *btn_info)
{
	dlog_print(DLOG_INFO, LOG_TAG, "Reset button is clicked !!");

	/* Initialize each variables */
	data_set_value_on_reset();

	/* Display '|' when no formula is there */
	view_set_text_to_label("|");
}

/**
 * @brief Deletes the last one of the displayed formula.
 * @param[in] btn_info Information of the pressed button
 */
static void _extra_del(button_info_s *btn_info)
{
	data_control_del_button(btn_info);
}

/**
 * @brief Adds the blank to the displayed formula.
 * @param[in] btn_info Information of the pressed button
 */
static void _extra_blank(button_info_s *btn_info)
{
	data_add_blank(btn_info);
}

/**
 * @brief Adds a decimal point to the displayed formula.
 * @param[in] btn_info Information of the pressed button
 */
static void _extra_decimal(button_info_s *btn_info)
{
	data_add_decimal_point(btn_info);
}

/**
 * @brief Adds the mark whether the number is positive or negative.
 * @param[in] btn_info Information of the pressed button
 */
static void _extra_pl_mi(button_info_s *btn_info)
{
	dlog_print(DLOG_INFO, LOG_TAG, "Pl_Mi button is clicked !!");
}

/**
 * @brief Calculates the whole of the formula and displays the result.
 * @param[in] btn_info Information of the pressed button
 */
static void _extra_result(button_info_s *btn_info)
{
	data_calculate_result(btn_info);
}

/**
 * @brief Function will be called when the button except number is clicked.
 * @param[in] btn_info Information of the pressed button
 */
static void _add_extra(button_info_s *btn_info)
{
	dlog_print(DLOG_INFO, LOG_TAG, "Extra button is clicked");
	switch (btn_info->button_val) {
	case KEY_VAL_RE:
		_extra_reset(btn_info);
		break;
	case KEY_VAL_DEL:
		_extra_del(btn_info);
		break;
	case KEY_VAL_BLANK:
		_extra_blank(btn_info);
		break;
	case KEY_VAL_PL_MI:
		_extra_pl_mi(btn_info);
		break;
	case KEY_VAL_DECIMAL:
		_extra_decimal(btn_info);
		break;
	case KEY_VAL_RESULT:
		_extra_result(btn_info);
		break;
	default:
		break;
	}
	/* Just display '|', after reset button is clicked */
	if (btn_info->button_val == KEY_VAL_RE) {
	} else {
		struct formula_info_s *formula_info = NULL;

		formula_info = data_get_formula_info();
		formula_info->formula_len = data_get_len_of_formula(formula_info->result_formula);

		view_display_formula(formula_info->displayed_formula, formula_info->formula_len);
	}
}

/**
 * @brief Function will be called when the button is clicked determining
 * what kind of button is clicked.
 * @param[in] data Information of the clicked button
 * @param[in] obj Clicked button
 * @param[in] event_info Information of the clicked event
 */
static void _button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	button_info_s *btn_info = (button_info_s *) data;
	dlog_print(DLOG_INFO, LOG_TAG, "button is clicked [ type: %d, %s ]", btn_info->button_type, btn_info->button_val_str);

	switch (btn_info->button_type) {
	case KEY_TYPE_EX:
		_add_extra(btn_info);
		break;
	case KEY_TYPE_NUM:
		_add_number(btn_info);
		break;
	case KEY_TYPE_OP:
		_add_operand(btn_info);
		break;
	default:
		break;
	}

	if (btn_info->button_type != KEY_TYPE_EX) {
		struct formula_info_s *formula_info = NULL;
		int formula_len = 0;

		formula_info = data_get_formula_info();
		formula_len = data_get_len_of_formula(formula_info->result_formula);

		view_display_formula(formula_info->displayed_formula, formula_len);
	}
}

/**
 * @brief Hook to take necessary actions before main event loop starts.
 * @param[in] user_data The user data to be passed to the callback function
 * Initialize UI resources and application's data
 * If this function returns true, the main loop of application starts
 * If this function returns false, the application is terminated
 */
static bool app_create(void *user_data)
{
	int i;
	char file_path[PATH_MAX] = { 0, };

	/* Get the path of EDJ file */
	data_get_resource_path(EDJ_FILE, file_path, sizeof(file_path));

	/* Initialize the new theme using EDJ file */
	view_init_calc_theme(file_path);

	/* Create window, conformant */
	view_create();

	/* Create specialized layout for the calculator using EDJ file */
	view_create_calc_layout(file_path, GRP_MAIN);

	/* Create label display the formula on the screen */
	view_create_calc_label();

	/* Create each button for the calculator */
	for (i = 0; i < CALC_BUTTON; i++) {
		view_create_calc_button(&BUTTON_PARAM[i], _button_clicked_cb);
	}

	return true;
}

/**
 * @brief This callback function is called when another application.
 * @param[in] app_control The handle to the app_control
 * @param[in] user_data The user data to be passed to the callback function
 * sends the launch request to the application
 */
static void app_control(app_control_h app_control, void *user_data)
{
	/* Handle the launch request. */
}

/**
 * @brief This callback function is called each time.
 * @param[in] user_data The user data to be passed to the callback function
 * the application is completely obscured by another application
 * and becomes invisible to the user
 */
static void app_pause(void *user_data)
{
	/* Take necessary actions when application becomes invisible. */
}

/**
 * @brief This callback function is called each time.
 * @param[in] user_data The user data to be passed to the callback function
 * the application becomes visible to the user
 */
static void app_resume(void *user_data)
{
	/* Take necessary actions when application becomes visible. */
}

/**
 * @brief This callback function is called once after the main loop of the application exits.
 * @param[in] user_data The user data to be passed to the callback function
 */
static void app_terminate(void *user_data)
{
	/* Release all resources. */
	char file_path[PATH_MAX] = {0, };

	/* Get the EDJ file path */
	data_get_resource_path(EDJ_FILE, file_path, sizeof(file_path));

	/* Finalize the theme using EDJ file */
	view_fini_calc_theme(file_path);

	/* Destroy the window */
	view_destroy();
}

/**
 * @brief This function will be called when the language is changed.
 * @param[in] event_info The system event information
 * @param[in] user_data The user data to be passed to the callback function
 */
static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;

	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);

	if (locale != NULL) {
		elm_language_set(locale);
		free(locale);
	}
	return;
}

/**
 * @brief Main function of the application.
 * @param[in] argc The argument count
 * @param[in] argv The argument vector
 */
int main(int argc, char *argv[])
{
	int ret;

	ui_app_lifecycle_callback_s event_callback = {0, };
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	/*
	 * If you want to handling more events,
	 * Please check the application lifecycle guide
	 */
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, NULL);

	ret = ui_app_main(argc, argv, &event_callback, NULL);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() is failed. err = %d", ret);

	return ret;
}
